#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import re
import time
from logging.handlers import TimedRotatingFileHandler
from pythonjsonlogger import jsonlogger
from datetime import datetime

def timeit(logger, session):
	def timeit_decorator(method):
	    def timed(*args, **kw):
	        ts = time.time()
	        result = method(*args, **kw)
	        te = time.time()
	        logger.debug('Time (%s): %2.4f sec' % (method.__name__.upper(), te-ts), session=session)
	        return result
	    return timed
	return timeit_decorator

RESET_SEQ = "\033[0m"
COLOR_SEQ = "\033[1;%dm"
BOLD_SEQ = "\033[1m"
BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)
COLORS = {
    'WARNING': YELLOW,
    'INFO': WHITE,
    'DEBUG': CYAN,
    'CRITICAL': RED,
    'ERROR': RED
}

class ColoredFormatter(logging.Formatter):
    def __init__(self, msg, use_color = True):
        logging.Formatter.__init__(self, msg)
        self.use_color = use_color

    def format(self, record):
        levelname = record.levelname
        if self.use_color and levelname in COLORS:
            levelname_color = COLOR_SEQ % (30 + COLORS[levelname]) + levelname + RESET_SEQ
            record.levelname = levelname_color
        return logging.Formatter.format(self, record)

def formatter_message(message, use_color = True):
    if use_color:
        message = message.replace("$RESET", RESET_SEQ).replace("$BOLD", BOLD_SEQ)
    else:
        message = message.replace("$RESET", "").replace("$BOLD", "")
    return message

class BLJsonFormatter(jsonlogger.JsonFormatter):
    def parse(self):
        return self._fmt.split(';')
    def add_fields(self, log_record, record, message_dict):
        super(BLJsonFormatter, self).add_fields(log_record, record, message_dict)
        if not log_record.get('timestamp'):
            # this doesn't use record.created, so it is slightly off
            now = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%S.%fZ')
            log_record['@timestamp'] = now
        if log_record.get('level'):
            log_record['severity'] = log_record['level'].upper()
        else:
            log_record['severity'] = record.levelname

class Logger():
	def __init__(self, name, file, env='local'):
		logger = logging.getLogger(name)
		reporter = logging.getLogger(name + '.reporter')
		
		if len(logger.handlers) != 1:
			logger.propagate = False
			logger.setLevel(logging.DEBUG)
			if env == "local":
				handler = logging.StreamHandler()
				handler.setLevel(logging.DEBUG)
				formatter = ColoredFormatter(formatter_message('$BOLD%(levelname)-19s$RESET - %(name)s - %(message)s', True))
			else:
				handler = TimedRotatingFileHandler(file, when="midnight", interval=1)
				handler.suffix = "%Y%m%d"
				handler.extMatch = re.compile(r"^\d{8}$")
				handler.setLevel(logging.WARNING)
				formatter = BLJsonFormatter(u'@timestamp;level;name;message')
			
			handler.setFormatter(formatter)
			logger.addHandler(handler)

		if len(reporter.handlers) != 1:
			reporter.setLevel(logging.DEBUG)
			if env == "local":
				r_handler = logging.StreamHandler()
				r_handler.setLevel(logging.ERROR)
				formatter = ColoredFormatter(formatter_message('%(asctime)s - $BOLD%(levelname)-19s$RESET - %(name)s - %(message)s', True))
			else:
				r_handler = TimedRotatingFileHandler(file, when="midnight", interval=1)
				r_handler.suffix = "%Y%m%d"
				r_handler.extMatch = re.compile(r"^\d{8}$")
				r_handler.setLevel(logging.INFO)
				formatter = BLJsonFormatter(u'@timestamp;level;name;message')
			
			r_handler.setFormatter(formatter)
			reporter.addHandler(r_handler)

		self.env = env
		self.logger = logger
		self.reporter = reporter

	def debug(self, message, session="undefined"):
		if self.env == "local":
			self.logger.debug(session + " - " + message)
		else:
			self.logger.debug(message, extra={"session": session})

	def info(self, message, session="undefined"):
		if self.env == "local":
			self.logger.info(session + " - " + message)
		else:
			self.logger.info(message, extra={"session": session})

	def warning(self, message, session="undefined"):
		if self.env == "local":
			self.logger.warning(session + " - " + message)
		else:
			self.logger.warning(message, extra={"session": session})

	def error(self, e, session="undefined"):
		if self.env == "local":
			self.logger.fatal(session + "%s" % e, exc_info=True)
		else:
			self.logger.fatal(e, exc_info=True, extra={"session": session})

	def report(self, body):
		self.reporter.info("{0} - {1} * {2} ({3}) - {4}".format(body["session"], body["query"], body["intent"], body["confidence"], body["status"]), extra=body)
